#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>
#include "ExpressionLexer.h"
#include "Operand.h"

//Private functions

SymbolType classifyToken(Token t)
{
    SymbolType type = None;
    switch (t[0])
    {
        case '+':
            type = OperatorPlus;
            break;
        case '-':
            type = OperatorMinus;
            break;
        case '*':
            type = OperatorMul;
            break;
        case '/':
            type = OperatorDiv;
            break;
        case '(':
            type = ParOpen;
            break;
        case ')':
            type = ParClose;
            break;
        default:
            type = isOperand(t) ? Operand : Other;
            break;
    }
    return type;
}

//Public functions

ExpressionLexer* newExpressionLexer(char expression[])
{
    ExpressionLexer *lexer = (ExpressionLexer*)malloc(sizeof(ExpressionLexer));
    lexer->tokenizer = newTokenizer(expression, " ");
    lexer->currentTokenType = None;
    return lexer;
}

void destroyExpressionLexer(ExpressionLexer *lexer)
{
    destroyTokenizer(lexer->tokenizer);
    free(lexer);
}

boolean readNextTokenFromLex(ExpressionLexer* lexer)
{
    //Lettura token dal tokenizer
    boolean result = readNextTokenFromTk(lexer->tokenizer);
    if (result) 
    {   //Recupero e classificazione token
        Token currentToken;
        getCurrentTokenFromTk(lexer->tokenizer, currentToken);
        lexer->currentTokenType = classifyToken(currentToken);
    }
    return result;
}

void getCurrentTokenFromLex(ExpressionLexer* lexer, Token t)
{
    getCurrentTokenFromTk(lexer->tokenizer, t);
}

SymbolType getCurrentTokenType(ExpressionLexer* lexer)
{
    return lexer->currentTokenType;
}