#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "ExtendedTokenizer.h"

ExtendedTokenizer* newTokenizer(char expression[], char tokens[], char ignores[])
{
    ExtendedTokenizer *tokenizer = (ExtendedTokenizer*)malloc(sizeof(ExtendedTokenizer));

    assert(strlen(expression) < EXP_BUFFER_DIM);
    strcpy(tokenizer->buffer, expression);
    assert(strlen(tokens) < CONTROL_CHAR_DIM); //altri controlli?
    strcpy(tokenizer->tokenChars, tokens);
    assert(strlen(ignores) < CONTROL_CHAR_DIM);//altri controlli?
    strcpy(tokenizer->ignoreChars, ignores);
    tokenizer->currentIdx = 0;
    strcpy(tokenizer->currentToken, "");
    return tokenizer;
}

void destroyTokenizer(ExtendedTokenizer *tokenizer)
{
    free(tokenizer);
}

char getCurrentChar(ExtendedTokenizer *tokenizer)
{
    return tokenizer->buffer[tokenizer->currentIdx];
}

void skipCurrentChar(ExtendedTokenizer * tokenizer)
{
    tokenizer->currentIdx++;
}

boolean isCurrentExpressionEnd(ExtendedTokenizer *tokenizer)
{
    return getCurrentChar(tokenizer) == '\0';
}

boolean isCurrentTokenChar(ExtendedTokenizer *tokenizer)
{
    if (isCurrentExpressionEnd(tokenizer))
    {
        return false;
    }
    else
    {        
        return strchr(tokenizer->tokenChars, 
            getCurrentChar(tokenizer)) != NULL;
    }
}

boolean isCurrentIgnoreChar(ExtendedTokenizer *tokenizer)
{
    if (isCurrentExpressionEnd(tokenizer))
    {
        return false;
    }
    else
    {        
        return strchr(tokenizer->ignoreChars, 
            getCurrentChar(tokenizer)) != NULL;
    }
}

void initCurrentToken(ExtendedTokenizer *tokenizer)
{
    tokenizer->currentToken[0] = '\0';
}

void addCurrentCharToCurrentToken(ExtendedTokenizer *tokenizer)
{
    int len = strlen(tokenizer->currentToken);
    tokenizer->currentToken[len] = getCurrentChar(tokenizer);
    tokenizer->currentToken[len + 1] = '\0';
}

boolean readNextTokenFromTk(ExtendedTokenizer* tokenizer)
{   
    initCurrentToken(tokenizer);
    while (!isCurrentExpressionEnd(tokenizer) &&
        isCurrentIgnoreChar(tokenizer))    //expression trimLeft
    {
        skipCurrentChar(tokenizer);
    }
    if (isCurrentExpressionEnd(tokenizer))  //siamo arrivati in fondo?
    {
        return false;
    }
    if (isCurrentTokenChar(tokenizer))  // un token?
    {
        addCurrentCharToCurrentToken(tokenizer);
        skipCurrentChar(tokenizer);
        return true;
    }
    do
    {
        addCurrentCharToCurrentToken(tokenizer);
        skipCurrentChar(tokenizer);
    }
    while (!isCurrentTokenChar(tokenizer) &&
        !isCurrentIgnoreChar(tokenizer) &&
        !isCurrentExpressionEnd(tokenizer));
    return true;
}

void getCurrentTokenFromTk(ExtendedTokenizer* tokenizer, Token t)
{
    strcpy(t, tokenizer->currentToken);
}

void tokenizerTest()
{
    Token t;
    ExtendedTokenizer *tizer = newTokenizer("10+ 3   / 1234 (  ", "+/(", " ");
    
    getCurrentTokenFromTk(tizer, t);
    assert(t[0] == '\0');

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Number);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "10") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Symbol);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "+") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Number);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "3") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Symbol);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "/") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Number);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "1234") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Symbol);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "(") == 0);

    assert(!readNextTokenFromTk(tizer));
}

