#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "SimpleTokenizer.h"

SimpleTokenizer* newTokenizer(char expression[], char ignores[])
{
    SimpleTokenizer *tokenizer = (SimpleTokenizer*)malloc(sizeof(SimpleTokenizer));

    assert(strlen(expression) < EXP_BUFFER_DIM);
    strcpy(tokenizer->buffer, expression);
    assert(strlen(ignores) < CONTROL_CHAR_DIM);//altri controlli?
    strcpy(tokenizer->ignoreChars, ignores);
    tokenizer->currentIdx = 0;
    strcpy(tokenizer->currentToken, "");
    return tokenizer;
}

void destroyTokenizer(SimpleTokenizer *tokenizer)
{
    free(tokenizer);
}

char getCurrentChar(SimpleTokenizer *tokenizer)
{
    return tokenizer->buffer[tokenizer->currentIdx];
}

void skipCurrentChar(SimpleTokenizer * tokenizer)
{
    tokenizer->currentIdx++;
}

boolean isCurrentExpressionEnd(SimpleTokenizer *tokenizer)
{
    return getCurrentChar(tokenizer) == '\0';
}

boolean isCurrentIgnoreChar(SimpleTokenizer *tokenizer)
{
    if (isCurrentExpressionEnd(tokenizer))
    {
        return false;
    }
    else
    {        
        return strchr(tokenizer->ignoreChars, 
            getCurrentChar(tokenizer)) != NULL;
    }
}

void initCurrentToken(SimpleTokenizer *tokenizer)
{
    tokenizer->currentToken[0] = '\0';
}

void addCurrentCharToCurrentToken(SimpleTokenizer *tokenizer)
{
    int len = strlen(tokenizer->currentToken);
    tokenizer->currentToken[len] = getCurrentChar(tokenizer);
    tokenizer->currentToken[len + 1] = '\0';
}

boolean readNextTokenFromTk(SimpleTokenizer* tokenizer)
{   
    initCurrentToken(tokenizer);
    while (!isCurrentExpressionEnd(tokenizer) &&
        isCurrentIgnoreChar(tokenizer))    //expression trimLeft
    {
        skipCurrentChar(tokenizer);
    }
    if (isCurrentExpressionEnd(tokenizer))  //siamo arrivati in fondo?
    {
        return false;
    }
    do
    {
        addCurrentCharToCurrentToken(tokenizer);
        skipCurrentChar(tokenizer);
    }
    while (!isCurrentIgnoreChar(tokenizer) &&
        !isCurrentExpressionEnd(tokenizer));
    return true;
}

void getCurrentTokenFromTk(SimpleTokenizer* tokenizer, Token t)
{
    strcpy(t, tokenizer->currentToken);
}

void tokenizerTest()
{
    Token t;
    SimpleTokenizer *tizer = newTokenizer("10+ 3   / 1234(  ", " ");
    
    getCurrentTokenFromTk(tizer, t);
    assert(t[0] == '\0');

    assert(readNextTokenFromTk(tizer));
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "10+") == 0);

    assert(readNextTokenFromTk(tizer));
    //assert(getCurrentTokenType(tizer) == Number);
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "3") == 0);

    assert(readNextTokenFromTk(tizer));
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "/") == 0);

    assert(readNextTokenFromTk(tizer));
    getCurrentTokenFromTk(tizer, t);
    assert(strcmp(t, "1234(") == 0);
    assert(!readNextTokenFromTk(tizer));
}

